@@ -1,23 +1,38 @@
-extern crate clap;
-#[macro_use]
-extern crate serde_derive;
-extern crate termsize;
-
 use chrono::prelude::*;
 use clap::Parser;
-use regex::Regex;
 
 use crate::minute::get_minute;
 
 mod formatter;
 mod minute;
 
-static COLOUR_HELP: & str = "Available colours are black, red, green, yellow, blue, magenta, cyan, white.\nEach colour can be prefixed with 'bright-'.";
+static COLOURS: &[&str; 16] = &[
+    "bright-black",
+    "bright-blue",
+    "bright-cyan",
+    "bright-green",
+    "bright-magenta",
+    "bright-red",
+    "bright-white",
+    "bright-yellow",
+    "black",
+    "blue",
+    "cyan",
+    "green",
+    "magenta",
+    "red",
+    "white",
+    "yellow",
+];
+
+static COLOUR_HELP: &str = "Available colours are black, blue, cyan, green, magenta, red, white and yellow.\nEach colour can be prefixed with 'bright-'.";
 static DEFAULT_MAIN: &str = "bright-black";
 static DEFAULT_TIME: &str = "red";
 static DEFAULT_AUTHOR: &str = "white";
 static MIN_WIDTH: usize = 20;
 static MAX_WIDTH: usize = 120;
+static DEFAULT_WIDTH: usize = 80; // If we fail to get terminal width
+static MARGIN: usize = 5; // Margin on the right of the comment, only used when automatically detecting the terminal width
 
 #[derive(Parser, Debug)]
 #[clap(version, about)]
@@ -52,23 +67,23 @@ struct Args {
 fn main() {
     let args = Args::parse();
 
-    let local: DateTime<Local> = Local::now();
-    let now = format!(
-        "{:0width$}:{:0width$}",
-        local.hour(),
-        local.minute(),
-        width = 2
-    );
-
-    let timestamp = args.time.unwrap_or(now);
+    let timestamp = args.time.unwrap_or_else(|| {
+        let local: DateTime<Local> = Local::now();
+        format!(
+            "{:0width$}:{:0width$}",
+            local.hour(),
+            local.minute(),
+            width = 2
+        )
+    });
     let minute = get_minute(&timestamp);
-    let max_width = args.max_width as u16;
+    let max_width = args.max_width;
 
-    let width: usize = args.width.unwrap_or(
+    let width: usize = args.width.unwrap_or_else(|| {
         termsize::get()
-            .map(|size| std::cmp::min(size.cols, max_width) - 10)
-            .unwrap() as usize,
-    );
+            .map(|size| std::cmp::min(size.cols as usize, max_width) - MARGIN)
+            .unwrap_or(DEFAULT_WIDTH)
+    });
 
     print!(
         "{}",
@@ -82,19 +97,16 @@ fn main() {
 }
 
 fn is_timestamp(val: &str) -> Result<(), String> {
-    let re = Regex::new(r"^([01][0-9]|2[0-3]):[0-5][0-9]$").unwrap();
-    if re.is_match(val) {
-        Ok(())
-    } else {
-        Err(String::from(
+    match NaiveTime::parse_from_str(val, "%H:%M") {
+        Ok(_) => Ok(()),
+        _ => Err(String::from(
             "The value must be a valid 24-hour timestamp in the format HH:MM",
-        ))
+        )),
     }
 }
 
 fn is_colour(val: &str) -> Result<(), String> {
-    let re = Regex::new(r"^(bright-)?(black|red|green|yellow|blue|magenta|cyan|white)$").unwrap();
-    if re.is_match(val) {
+    if COLOURS.iter().any(|&c| c == val) {
         Ok(())
     } else {
         Err(format!("Unknown colour.\n{}", COLOUR_HELP))
